@@ -110,6 +110,7 @@ struct btd_device {
 	bdaddr_t	bdaddr;
 	gchar		*path;
 	char		name[MAX_NAME_LENGTH + 1];
+	char		*alias;
 	struct btd_adapter	*adapter;
 	GSList		*uuids;
 	GSList		*drivers;		/* List of driver_data */
@@ -134,6 +135,7 @@ struct btd_device {
 
 	sdp_list_t	*tmp_records;
 
+	gboolean	trusted;
 	gboolean	paired;
 	gboolean	renewed_key;
 
@@ -191,6 +193,8 @@ static void browse_request_free(struct browse_req *req)
 		dbus_message_unref(req->msg);
 	if (req->conn)
 		dbus_connection_unref(req->conn);
+	if (req->device)
+		btd_device_unref(req->device);
 	g_slist_foreach(req->profiles_added, (GFunc) g_free, NULL);
 	g_slist_free(req->profiles_added);
 	g_slist_free(req->profiles_removed);
@@ -212,8 +216,8 @@ static void browse_request_cancel(struct browse_req *req)
 
 	bt_cancel_discovery(&src, &device->bdaddr);
 
-	browse_request_free(req);
 	device->browse = NULL;
+	browse_request_free(req);
 }
 
 static void device_free(gpointer user_data)
@@ -246,6 +250,7 @@ static void device_free(gpointer user_data)
 
 	g_free(device->authr);
 	g_free(device->path);
+	g_free(device->alias);
 	g_free(device);
 }
 
@@ -254,6 +259,11 @@ gboolean device_is_paired(struct btd_device *device)
 	return device->paired;
 }
 
+gboolean device_is_trusted(struct btd_device *device)
+{
+	return device->trusted;
+}
+
 static DBusMessage *get_properties(DBusConnection *conn,
 				DBusMessage *msg, void *user_data)
 {
@@ -298,13 +308,12 @@ static DBusMessage *get_properties(DBusConnection *conn,
 	dict_append_entry(&dict, "Name", DBUS_TYPE_STRING, &ptr);
 
 	/* Alias (fallback to name or address) */
-	if (read_device_alias(srcaddr, dstaddr, name, sizeof(name)) < 1) {
-		if (strlen(ptr) == 0) {
-			g_strdelimit(dstaddr, ":", '-');
-			ptr = dstaddr;
-		}
-	} else
-		ptr = name;
+	if (device->alias != NULL)
+		ptr = device->alias;
+	else if (strlen(ptr) == 0) {
+		g_strdelimit(dstaddr, ":", '-');
+		ptr = dstaddr;
+	}
 
 	dict_append_entry(&dict, "Alias", DBUS_TYPE_STRING, &ptr);
 
@@ -324,7 +333,7 @@ static DBusMessage *get_properties(DBusConnection *conn,
 	dict_append_entry(&dict, "Paired", DBUS_TYPE_BOOLEAN, &boolean);
 
 	/* Trusted */
-	boolean = read_trust(&src, dstaddr, GLOBAL_TRUST);
+	boolean = device_is_trusted(device);
 	dict_append_entry(&dict, "Trusted", DBUS_TYPE_BOOLEAN, &boolean);
 
 	/* Connected */
@@ -357,6 +366,11 @@ static DBusMessage *set_alias(DBusConnection *conn, DBusMessage *msg,
 	bdaddr_t src;
 	int err;
 
+	/* No change */
+	if ((device->alias == NULL && g_str_equal(alias, "")) ||
+			g_strcmp0(device->alias, alias) == 0)
+		return dbus_message_new_method_return(msg);
+
 	adapter_get_address(adapter, &src);
 	ba2str(&src, srcaddr);
 	ba2str(&device->bdaddr, dstaddr);
@@ -369,6 +383,9 @@ static DBusMessage *set_alias(DBusConnection *conn, DBusMessage *msg,
 				ERROR_INTERFACE ".Failed",
 				strerror(-err));
 
+	g_free(device->alias);
+	device->alias = g_str_equal(alias, "") ? NULL : g_strdup(alias);
+
 	emit_property_changed(conn, dbus_message_get_path(msg),
 				DEVICE_INTERFACE, "Alias",
 				DBUS_TYPE_STRING, &alias);
@@ -377,18 +394,28 @@ static DBusMessage *set_alias(DBusConnection *conn, DBusMessage *msg,
 }
 
 static DBusMessage *set_trust(DBusConnection *conn, DBusMessage *msg,
-					dbus_bool_t value, void *data)
+					gboolean value, void *data)
 {
 	struct btd_device *device = data;
 	struct btd_adapter *adapter = device->adapter;
 	char srcaddr[18], dstaddr[18];
 	bdaddr_t src;
+	int err;
+
+	if (device->trusted == value)
+		return dbus_message_new_method_return(msg);
 
 	adapter_get_address(adapter, &src);
 	ba2str(&src, srcaddr);
 	ba2str(&device->bdaddr, dstaddr);
 
-	write_trust(srcaddr, dstaddr, GLOBAL_TRUST, value);
+	err = write_trust(srcaddr, dstaddr, GLOBAL_TRUST, value);
+	if (err < 0)
+		return g_dbus_create_error(msg,
+				ERROR_INTERFACE ".Failed",
+				strerror(-err));
+
+	device->trusted = value;
 
 	emit_property_changed(conn, dbus_message_get_path(msg),
 				DEVICE_INTERFACE, "Trusted",
@@ -842,7 +869,7 @@ struct btd_device *device_create(DBusConnection *conn,
 	struct btd_device *device;
 	const gchar *adapter_path = adapter_get_path(adapter);
 	bdaddr_t src;
-	char srcaddr[18];
+	char srcaddr[18], alias[MAX_NAME_LENGTH + 1];
 
 	device = g_try_malloc0(sizeof(struct btd_device));
 	if (device == NULL)
@@ -867,6 +894,9 @@ struct btd_device *device_create(DBusConnection *conn,
 	adapter_get_address(adapter, &src);
 	ba2str(&src, srcaddr);
 	read_device_name(srcaddr, address, device->name);
+	if (read_device_alias(srcaddr, address, alias, sizeof(alias)) == 0)
+		device->alias = g_strdup(alias);
+	device->trusted = read_trust(&src, address, GLOBAL_TRUST);
 
 	device->auth = 0xff;
 
@@ -879,9 +909,6 @@ struct btd_device *device_create(DBusConnection *conn,
 void device_set_name(struct btd_device *device, const char *name)
 {
 	DBusConnection *conn = get_dbus_connection();
-	char alias[MAX_NAME_LENGTH + 1];
-	char srcaddr[18], dstaddr[18];
-	bdaddr_t src;
 
 	if (strncmp(name, device->name, MAX_NAME_LENGTH) == 0)
 		return;
@@ -892,11 +919,7 @@ void device_set_name(struct btd_device *device, const char *name)
 				DEVICE_INTERFACE, "Name",
 				DBUS_TYPE_STRING, &name);
 
-	adapter_get_address(device->adapter, &src);
-	ba2str(&src, srcaddr);
-	ba2str(&device->bdaddr, dstaddr);
-
-	if (read_device_alias(srcaddr, dstaddr, alias, sizeof(alias)) == 0)
+	if (device->alias != NULL)
 		return;
 
 	emit_property_changed(conn, device->path,
@@ -1311,16 +1334,29 @@ static void store_profiles(struct btd_device *device)
 	g_free(str);
 }
 
+static void create_device_reply(struct btd_device *device, struct browse_req *req)
+{
+	DBusMessage *reply;
+
+	reply = dbus_message_new_method_return(req->msg);
+	if (!reply)
+		return;
+
+	dbus_message_append_args(reply, DBUS_TYPE_OBJECT_PATH, &device->path,
+					DBUS_TYPE_INVALID);
+
+	g_dbus_send_message(req->conn, reply);
+}
+
 static void search_cb(sdp_list_t *recs, int err, gpointer user_data)
 {
 	struct browse_req *req = user_data;
 	struct btd_device *device = req->device;
-	DBusMessage *reply;
 
 	if (err < 0) {
 		error("%s: error updating services: %s (%d)",
 				device->path, strerror(-err), -err);
-		goto proceed;
+		goto send_reply;
 	}
 
 	update_services(req, recs);
@@ -1334,7 +1370,7 @@ static void search_cb(sdp_list_t *recs, int err, gpointer user_data)
 
 	if (!req->profiles_added && !req->profiles_removed) {
 		debug("%s: No service update", device->path);
-		goto proceed;
+		goto send_reply;
 	}
 
 	/* Probe matching drivers for services added */
@@ -1348,34 +1384,32 @@ static void search_cb(sdp_list_t *recs, int err, gpointer user_data)
 	/* Propagate services changes */
 	services_changed(req->device);
 
-proceed:
-	/* Store the device's profiles in the filesystem */
-	store_profiles(device);
-
+send_reply:
 	if (!req->msg)
 		goto cleanup;
 
 	if (dbus_message_is_method_call(req->msg, DEVICE_INTERFACE,
-					"DiscoverServices")) {
+					"DiscoverServices"))
 		discover_services_reply(req, err, req->records);
-		goto cleanup;
+	else if (dbus_message_is_method_call(req->msg, ADAPTER_INTERFACE,
+						"CreatePairedDevice"))
+		create_device_reply(device, req);
+	else if (dbus_message_is_method_call(req->msg, ADAPTER_INTERFACE,
+						"CreateDevice")) {
+		if (err < 0) {
+			error_failed_errno(req->conn, req->msg, -err);
+			goto cleanup;
+		}
+
+		create_device_reply(device, req);
+		device_set_temporary(device, FALSE);
 	}
 
-	/* Reply create device request */
-	reply = dbus_message_new_method_return(req->msg);
-	if (!reply)
-		goto cleanup;
-
-	dbus_message_append_args(reply, DBUS_TYPE_OBJECT_PATH, &device->path,
-							DBUS_TYPE_INVALID);
-
-	g_dbus_send_message(req->conn, reply);
-
-	device_set_temporary(device, FALSE);
-
 cleanup:
-	browse_request_free(req);
+	if (!device->temporary)
+		store_profiles(device);
 	device->browse = NULL;
+	browse_request_free(req);
 }
 
 static void browse_cb(sdp_list_t *recs, int err, gpointer user_data)
@@ -1449,7 +1483,7 @@ int device_browse(struct btd_device *device, DBusConnection *conn,
 		conn = get_dbus_connection();
 
 	req->conn = dbus_connection_ref(conn);
-	req->device = device;
+	req->device = btd_device_ref(device);
 
 	if (search) {
 		memcpy(&uuid, search, sizeof(uuid_t));
@@ -1477,8 +1511,8 @@ int device_browse(struct btd_device *device, DBusConnection *conn,
 	err = bt_search_service(&src, &device->bdaddr,
 				&uuid, cb, req, NULL);
 	if (err < 0) {
-		browse_request_free(req);
 		device->browse = NULL;
+		browse_request_free(req);
 	}
 
 	return err;
